/*
 * $Id: CategoryLabel.cc,v 1.2 2003/02/21 03:33:53 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CategoryLabel.h"
#include "Category.h"

gboolean selectedCatLabelButtonPress_cb(GtkWidget *,
					GdkEventButton *,
					gpointer);
/** Constructor.

    \param c is the pointer to the Category.
 */
CategoryLabel::CategoryLabel(Category *c)
{
  g_assert(c != NULL);
  cat = c;
  textView = gtk_text_view_new_with_buffer(cat->getCatNameBuffer());
  g_signal_connect(G_OBJECT(textView),
		   "button-press-event",
		   G_CALLBACK(selectedCatLabelButtonPress_cb),
		   this);

  gtk_text_view_set_editable(GTK_TEXT_VIEW(textView), FALSE);
  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(textView), FALSE);
}

/** Destructor.
 */
CategoryLabel::~CategoryLabel()
{
}

/** Open the Category.
 */
void
CategoryLabel::openCategory() const
{
  cat->showWindow();
}

/** Maximize the priority of the Category tag.
 */
void
CategoryLabel::bringCategoryToFront() const
{
  gchar *fontName;
  GtkTextTag *tag;
  GtkTextTagTable *table;

  tag = cat->getTag();
  table = cat->getTagTable();

  // Maximize the priority so that the properties of this tag are
  // visible.
  gtk_text_tag_set_priority(tag,
			    gtk_text_tag_table_get_size(table) - 1);

  // Try to make the changes visible.  This is a workaround.
  g_object_get((gpointer) tag,
	       "font",
	       &fontName,
	       NULL);
  g_object_set((gpointer) tag,
	       "font",
	       fontName,
	       NULL);
  g_free(fontName);
}

// Callbacks.
/** Callback to respond to clicks on the Category label view.
 */
gboolean
selectedCatLabelButtonPress_cb(GtkWidget *widget,
			       GdkEventButton *event,
			       gpointer data)
{
  CategoryLabel *catLab = (CategoryLabel *) data;

  if (event->type == GDK_2BUTTON_PRESS)
    catLab->openCategory();
  else
    catLab->bringCategoryToFront();

  // Continue with normal handling.
  return(FALSE);
}
