/*
 * $Id: CategoryLabels.cc,v 1.3 2003/02/19 21:28:21 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CategoryLabels.h"

/** Constructor.
 */
CategoryLabels::CategoryLabels()
{
  categoryLabel.clear();
  hBox = gtk_hbox_new(FALSE, 5);
  put = FALSE;
}

/** Destructor.
 */
CategoryLabels::~CategoryLabels()
{
  map<Category *, CategoryLabel *>::iterator iCatLab;

  if (!put)
    gtk_widget_destroy(hBox);

  for (iCatLab = categoryLabel.begin();
       iCatLab != categoryLabel.end();
       iCatLab++)
    delete iCatLab->second;
}

/** Create a Category label.

    \param cat points to the Category the label is for.
*/
void
CategoryLabels::createCategoryLabel(Category *cat)
{
  g_assert(cat != NULL);

  // Has this Category already been added?
  if (categoryLabel.count(cat) != 0)
    return; // Yes.  Do nothing.

  // Create a label.
  categoryLabel[cat] = new CategoryLabel(cat);

  // Add to the hbox.
  gtk_box_pack_start(GTK_BOX(hBox),
		     categoryLabel[cat]->getTextView(),
                     FALSE,
		     FALSE,
		     0);

  // Show it.
  gtk_widget_show_all(hBox);
}

/** Delete a Category label.

    \param cat points to the Category the label is for.
*/
void
CategoryLabels::deleteCategoryLabel(Category *cat)
{
  g_assert(cat != NULL);

  // Is there a label for this Category?
  if (categoryLabel.count(cat) == 0)
    return; // No.  Do nothing.

  // Remove the text view from the box.
  gtk_container_remove(GTK_CONTAINER(hBox),
		       categoryLabel[cat]->getTextView());

  // Delete the object.
  delete categoryLabel[cat];

  // Remove from the container.
  categoryLabel.erase(cat);
}

/** Insert a Category Label.

    \param catLab points to the Category label.  The view widget in the
    Category label has a non-floating reference.

    Because the view widget has a non-floating reference, the widget
    is dereferenced after adding it to the parent widget.
*/
void
CategoryLabels::insertCategoryLabel(CategoryLabel *catLab)
{
  g_assert(catLab != NULL);

  // Has this Category already been added?
  if (categoryLabel.count(catLab->getCategory()) != 0)
    return; // Yes.

  // Create a label.
  categoryLabel[catLab->getCategory()] = catLab;

  // Add to the hbox.
  gtk_box_pack_start(GTK_BOX(hBox),
		     catLab->getTextView(),
                     FALSE,
		     FALSE,
		     0);

  // Unreference.
  g_object_unref((gpointer) catLab->getTextView());
}

/** Extract a Category Label.

    \param cat points to the Category the label is for.  The view
    widget in the Category label is given a non-floating reference.

    Because the view widget has a non-floating reference, the widget
    must be dereferenced after adding it to its new parent.
*/
CategoryLabel *
CategoryLabels::extractCategoryLabel(Category *cat)
{
  CategoryLabel *catLab;

  g_assert(cat != NULL);

  // Assert that the Category label exists.
  g_assert(categoryLabel.count(cat) != 0);

  // Extract the Category Label.
  catLab = categoryLabel[cat]; // Get address
  categoryLabel.erase(cat); // Remove from the container
  g_object_ref((gpointer)catLab->getTextView()); // Reference the view widget
  gtk_container_remove(GTK_CONTAINER(hBox),
		       catLab->getTextView()); // Remove from the container

  return(catLab);
}
