/*
 * $Id: CategoryLabels.h,v 1.2 2003/02/04 21:05:59 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef CATEGORYLABELS_H
#define CATEGORYLABELS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "CategoryLabel.h"
#include <map>
#include <gtk/gtk.h>
#include <gnome.h>

/** Class CategoryLabels

    This class implements the methods for a CategoryLabels

    This class stores the widgets necessary for displaying all the
    Category names for Coded Text that begins on a given line of a
    Document.  Methods are provided for adding or removing a widget,
    and creating or destroying a widget.

*/
class CategoryLabels
{
public:
  CategoryLabels();
  virtual ~CategoryLabels();

  void setPutStatus(gboolean status) { put = status; } ///< Set the put status.
  gboolean getPutStatus() const { return(put); } ///< Get the put status.
  GtkWidget *getHBox() const { return(hBox); } ///< Get the hbox.

  /// Get the number of Category labels.
  gint size() const { return(categoryLabel.size()); }

  void createCategoryLabel(Category *);
  void deleteCategoryLabel(Category *);

  void insertCategoryLabel(CategoryLabel *);
  CategoryLabel *extractCategoryLabel(Category *);

private:
  GtkWidget *hBox; ///< Box to hold the view widgets.
  map<Category *, CategoryLabel *> categoryLabel; ///< Labels for each Category
  gboolean put; ///< TRUE is the hBox widget was put into the Layout container
};

#endif
