/*
 * $Id: Component.h,v 1.3 2003/02/10 21:06:57 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef COMPONENT_H
#define COMPONENT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "QualResObject.h"

// Derived classes.
class Document;
class Category;
class Set;

/** Base class for Project components.
*/
class Component: public QualResObject
{
 public:

  Component();

  virtual ~Component() {} ///< Destructor

  void loadMap(map<const string, Component *> *, QualResObject::Type);
  void loadModel(GtkTreeStore *, QualResObject::Type);
  virtual void commitName();

  /** Hide the window. */
  virtual void hideWindow()
    { hideAllSelectWindows(); QualResObject::hideWindow(); }
  void hideSelectWindow(enum QualResObject::Type);
  void hideAllSelectWindows();

  /// Get the Component name labeled with type.
  virtual gchar *getLabeledName() const
    {g_strdup((getPrettyName()).c_str());}

 protected:
  void componentProjectIter(GtkTreeIter *);
  GtkTreeStore *selectDocStore; ///< Project Document tree model
  GtkWidget *selectDocView; ///< Project Document tree view
  GtkWidget *selectDocWindow; ///< Popup window for selecting a Document

  GtkTreeStore *selectCatStore; ///< Project Category tree model
  GtkWidget *selectCatView; ///< Project Category tree view
  GtkWidget *selectCatWindow; ///< Popup window for selecting a Category

  GtkTreeStore *selectSetStore; ///< Project Set tree model
  GtkWidget *selectSetView; ///< Project Set tree view
  GtkWidget *selectSetWindow; ///< Popup window for selecting a Set

  map<const string, Document *> *projectDocMap; ///< Project Documents
  map<const string, Category *> *projectCatMap; ///< Project Categories
  map<const string, Set *> *projectSetMap; ///< Project Sets

  gchar *xmlTag; ///< Pointer to XML tag for the Component.
};

#endif
