/*
 * $Id: DocCat.cc,v 1.5 2003/03/27 03:31:09 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "DocCat.h"
#include "Category.h"
#include "qualresUtility.h"
#include <fstream>

// Callbacks.
gboolean catListButtonPress_cb(GtkWidget *, GdkEventButton *, gpointer);
void docCatExport_cb(GtkWidget *, gpointer);
void catListSelect_cb(GtkTreeView *, gpointer);

/** Inform the object that a component's properties have changed.

    \param comp is the component
*/
void
DocCat::propertiesHaveChanged(QualResObject *comp)
{
  if (comp == NULL)
    return;

  if ((Category *) comp == selectedCat)
    setSelectedCategory(selectedCat->getName());
}


/** Get and load the name of the selected Category.

 */
void
DocCat::getAndSetSelectedCategory()
{
  GtkTreeSelection *selection;
  gchar *name;
  GtkTreeIter iter;

  // Get the selected Category.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(selectCatView));

  // Do nothing if there's no selection.
  if (!gtk_tree_selection_get_selected(selection, NULL, &iter))
    return;

  // Get the Category name.
  gtk_tree_model_get(GTK_TREE_MODEL(selectCatStore),
		     &iter,
		     QualResObject::FULL_NAME,
		     &name,
		     -1);

  // Load the Category and its name.
  setSelectedCategory(name);

  g_free(name);
}

/** Load the Category to Code.

    \param name is the name of the Category to Code

*/
void
DocCat::setSelectedCategory(string name)
{
  GtkTextIter start, end;
  GtkTextTag *tag;
  gchar *fontName;

  if (name == "")
    {
      selectedCat = NULL;
      gtk_text_buffer_set_text(selectedCatBuffer,
			       "",
			       -1);
      return;
    }

  // Load Category name, and apply Category tag to it.
  selectedCat = (*projectCatMap)[name];
  tag = selectedCat->getTag();
  gtk_text_buffer_set_text(selectedCatBuffer,
			   (selectedCat->getPrettyName()).c_str(),
			   -1);
  gtk_text_buffer_get_bounds(selectedCatBuffer,
			     &start,
			     &end);
  gtk_text_buffer_apply_tag(selectedCatBuffer,
			    tag,
			    &start,
			    &end);

  // Maximize the priority so that the properties of this tag are
  // visible.  This applies to all Documents.
  if ((objType == QualResObject::DOCUMENT) ||
      (objType == QualResObject::MEMO))
    gtk_text_tag_set_priority(tag,
			      gtk_text_tag_table_get_size(projTagTable) - 1);

  // Try to make the changes visible.  This is a workaround.
  g_object_get((gpointer) tag,
	       "font",
	       &fontName,
	       NULL);
  g_object_set((gpointer) tag,
	       "font",
	       fontName,
	       NULL);
  g_free(fontName);
}

/** Show the properties of the Selected Category.

Nothing is done if there is no Selected Category.
*/
void
DocCat::bringSelectedCategoryToFront()
{
  GtkTextTag *tag;
  gchar *fontName;

  if (selectedCat == NULL)
    return;

  // Maximize the priority so that the properties of this tag are
  // visible.  This applies to all Documents.
  if ((objType == QualResObject::DOCUMENT) ||
      (objType == QualResObject::MEMO))
    {
      tag = selectedCat->getTag();
      gtk_text_tag_set_priority(tag,
				gtk_text_tag_table_get_size(projTagTable) - 1);

      // Try to make the changes visible.  This is a workaround.
      g_object_get((gpointer) tag,
		   "font",
		   &fontName,
		   NULL);
      g_object_set((gpointer) tag,
		   "font",
		   fontName,
		   NULL);
      g_free(fontName);
    }
}

/** Select a Category for coding from a popup list.
*/
void
DocCat::selectCodeCategoryList()
{
  // Variables used in creating the window.
  GtkWidget *view, *vbox, *hbox, *scrolledView, *button;
  GtkAdjustment *adjustment;
  gint result;

  // Variables used in getting the selection.
  GtkTreeSelection *selection;
  gchar *name;
  GtkTreeIter iter;

  string sProp;

  // Create the Category selection window.
  if (selectCatWindow == NULL)
    {
      // Determine the prompt for the window.
      gchar *prompt;

      if ((getType() == QualResObject::DOCUMENT) ||
	  (getType() == QualResObject::CATEGORY))
	prompt = "Select a Category";
      else if ((getType() == QualResObject::MEMO) ||
	  (getType() == QualResObject::XREF))
	prompt = "Select a Cross-Reference";
      else if ((getType() == QualResObject::NARRATIVE_DOCUMENT) ||
	  (getType() == QualResObject::FRAME))
	prompt = "Select a Frame";
      else
	{
	  g_assert_not_reached();
	  return;
	}

      selectCatWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
      gtk_window_set_transient_for(GTK_WINDOW(selectCatWindow),
				   GTK_WINDOW(mainWindow));
      gtk_window_set_destroy_with_parent(GTK_WINDOW(selectCatWindow),
					 TRUE);
      g_signal_connect(G_OBJECT(selectCatWindow),
		       "delete-event",
		       G_CALLBACK(hide_cb),
		       NULL);
      gtk_window_set_title(GTK_WINDOW(selectCatWindow),
			   (getPrettyName()).c_str());
      gtk_window_set_default_size(GTK_WINDOW(selectCatWindow),
				  SELECT_COMPONENT_XSIZE,
				  SELECT_COMPONENT_YSIZE);

      // Create the widgets.
      selectCatView = createCatTreeView(selectCatStore);
      addNDocsColumn(selectCatView, NDOCS_COLUMN_NUMBER);

      g_signal_connect(G_OBJECT(selectCatView),
		       "button-press-event",
		       G_CALLBACK(catListButtonPress_cb),
		       this);

      g_signal_connect(G_OBJECT(selectCatView),
		       "cursor-changed",
		       G_CALLBACK(catListSelect_cb),
		       this);

      // Pack the vbox.
      vbox = gtk_vbox_new(FALSE, 5);
      gtk_box_pack_start(GTK_BOX(vbox),
			 gtk_label_new(_(prompt)),
			 FALSE, FALSE, 0);
      scrolledView = gtk_scrolled_window_new(NULL, NULL);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				     GTK_POLICY_AUTOMATIC,
				     GTK_POLICY_AUTOMATIC);
      gtk_container_add(GTK_CONTAINER(scrolledView), selectCatView);
      gtk_box_pack_start(GTK_BOX(vbox),
			 scrolledView,
			 TRUE, TRUE, 0);
				       
      // Add the button.
      hbox = gtk_hbutton_box_new();
      gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox),
				GTK_BUTTONBOX_SPREAD);

      button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
      gtk_signal_connect(GTK_OBJECT(button), "clicked",
			 GTK_SIGNAL_FUNC(hideData_cb), selectCatWindow);

      gtk_box_pack_start(GTK_BOX(hbox),
			 button,
			 FALSE, TRUE, 0);

      gtk_box_pack_start(GTK_BOX(vbox),
			 hbox,
			 FALSE, FALSE, 0);

      // Add the view to the selection widget.
      gtk_container_add(GTK_CONTAINER(selectCatWindow),
			vbox);

      // Scroll selectCatView all the way to the left.
      gtk_widget_show_all(selectCatWindow);
      adjustment =
	gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolledView));
      gtk_adjustment_set_value(adjustment, adjustment->lower);
      gtk_adjustment_value_changed(adjustment);
    }

  // Show the selection window.
  gtk_widget_show_all(selectCatWindow);
}

/** Export the buffer of a Component to a file.

    \param fileName is the name of the file

    The Component name and description are also exported
*/
void
DocCat::exportFile(string &fileName) const
{
  ofstream file;
  string tempDesc = getDesc();
  gchar *text;
  gint indx;

  file.open(fileName.c_str(), ios::out | ios::trunc);
  if (file.bad())
    {
      showMessage("Error opening file '" + fileName + "'",
		  GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  text = getLabeledName();
  file << text << endl << endl;
  g_free(text);

  // Add padding after each occurance of "\n" in the description.
  indx = 0;
  while ((indx = tempDesc.find(NEWLINE_CHAR, indx)) != string::npos)
    tempDesc.insert(++indx, EXPORT_DESC_PAD);
  file << "Description:  " << tempDesc << endl << endl;

  file << EXPORT_SEPARATOR;

  // Output text.
  text = getTextToPrint();
  file << text;

  file.close();
  g_free(text);
}

/***********************************************************************/
/* CALLBACKS */
/***********************************************************************/
/** Select a destination Category from a popup list.
 */
void selectCodeCategoryList_cb(GtkWidget* widget, gpointer data)
{
  DocCat *comp = (DocCat *) data;

  comp->selectCodeCategoryList();
}

/** Code/view the selected Category.
 */
void codeViewSelectedCat_cb(GtkWidget* widget, gpointer data)
{
  DocCat *comp = (DocCat *) data;
  Category *selectedCat;
  selectedCat = comp->getSelectedCategory();

  if (selectedCat != NULL)
    selectedCat->showWindow();
  else
    showMessage("You must first select a Category",
		GTK_MESSAGE_ERROR,
		comp->getMainWindow());
}

/** Callback to respond to multiple clicks on the Category selection list.
 */
gboolean
catListButtonPress_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  DocCat *docCat = (DocCat *) data;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  gchar *name;
  GtkTreeIter iter;

  // Click with button 1.  Choose the selected Category.
  if ((event->button == 1) && (event->type == GDK_2BUTTON_PRESS))
    {
      // Get the selected Category.
      selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));

      // There should be a selection.
      if (!gtk_tree_selection_get_selected(selection, NULL, &iter))
	return(FALSE); // Continue with normal handling.

      // Get the Category name.
      model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
      gtk_tree_model_get(model,
			 &iter,
			 QualResObject::FULL_NAME,
			 &name,
			 -1);

      // Load the Category and its name.
      docCat->setSelectedCategory(name);
      g_free(name);

      // Hide the window.
      docCat->hideSelectWindow(QualResObject::CATEGORY);
      return(TRUE);
    }

  // Continue with normal handling.
  return(FALSE);
}

/** User clicked on a row.
 */
void catListSelect_cb(GtkTreeView *treeview, gpointer data)
{
  DocCat *docCat = (DocCat *) data;

  docCat->getAndSetSelectedCategory();
}

/** Export text from a DocCat.
*/
void
docCatExport_cb(GtkWidget *widget, gpointer data)
{
  DocCat *docCat = (DocCat *) data;
  GtkWidget *fileSelect;
  string fileName;
  gint result;

  // Pop up file selection box.
   fileSelect = gtk_file_selection_new("Please select a file for export");
   
   // Display.
   gtk_widget_show(fileSelect);
   result = gtk_dialog_run(GTK_DIALOG(fileSelect));
   fileName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileSelect));
   gtk_widget_destroy(GTK_WIDGET(fileSelect));

  // Abort.
   if ((result != GTK_RESPONSE_OK) ||
       g_file_test(fileName.c_str(), G_FILE_TEST_IS_DIR))
     return;

   // Export.
   docCat->exportFile(fileName);
}

/** Callback to respond to clicks on the selected Category view.
 */
gboolean
selectedCatViewButtonPress_cb(GtkWidget *widget,
			      GdkEventButton *event,
			      gpointer data)
{
  DocCat *docCat = (DocCat *) data;

  docCat->bringSelectedCategoryToFront();

  // Continue with normal handling.
  return(FALSE);
}
