/*
 * $Id: DocCat.h,v 1.3 2003/02/21 03:33:53 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef DOC_CAT_H
#define DOC_CAT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Component.h"

/** Base class for the Document and Category classes.

    This class implements methods common to only the Document and Category
    classes.
*/
class DocCat: public Component
{
 public:
  /** Types of searches.

      The user specifies a string to search for, and what part of the
      text to place in the destination Category.
  */
  enum SearchStore
  {
    STRING = 0, ///< Only the search string.
    WORD, ///< The word containing the search string.
    SENTENCE, ///< The sentence containing the search string.
    /// The sentence containing the search string and two
    // sentences before and after.
    SENTENCE2,
    STATEMENT, ///< The paragraph containing the search string.
    BUFFER, ///< The full text of the document containing the search string.
    NSEARCH_STORE
  };

  /** Command to either Code or Uncode text.
   */
  enum CodeUncode
  {
    CODE = 0,
    UNCODE,
    N_CODE_UNCODE
  };

  DocCat() { selectedCat = NULL; } ///< Constructor

  virtual ~DocCat() {} ///< Destructor

  virtual void propertiesHaveChanged(QualResObject *);

  /// Get the selected Category.
  Category *getSelectedCategory() const { return(selectedCat); }

  /// Get the Project tag table.
  GtkTextTagTable *getTagTable() const { return(projTagTable); }


  void getAndSetSelectedCategory();
  void setSelectedCategory(string);
  void bringSelectedCategoryToFront();

  /// Code or uncode selected text.
  virtual void codeUncodeText(enum CodeUncode) = 0;

  /// Search for and code text into the selected Category.
  virtual void codeSearchText(string &, DocCat::SearchStore, Category *) = 0;

  /// Export the buffer to a file.
  virtual void exportFile(string &) const;

  // GUI methods.
  void selectCodeCategoryList();

 protected:
  Category *selectedCat; ///< Selected Category.
  GtkTextBuffer *selectedCatBuffer; ///< Text entry for selected Category.
  GtkTextTagTable *projTagTable; ///< Project tag table
};

// Callbacks.
void selectCodeCategoryList_cb(GtkWidget *, gpointer);
void codeViewSelectedCat_cb(GtkWidget *, gpointer);
void docCatExport_cb(GtkWidget *, gpointer);
gboolean selectedCatViewButtonPress_cb(GtkWidget *,
					      GdkEventButton *,
					      gpointer);
#endif
