/*
 * $Id: SaveObject.h,v 1.1.1.1 2002/12/31 03:12:45 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef SAVE_OBJECT_H
#define SAVE_OBJECT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "qualres.h"
#include "PrintObject.h"

#include <string>
#include <libgdome/gdome.h>

/** Exception class.
 */
class FileException {};

/** Base class for objects that can save themselves to an XML element.

    All access functions of SaveObject and derived classes return a
    pointer to const.  For example, getError returns (const string *),
    pointing to the name field.  The object is responsible for this
    memory; the calling program should never free the memory pointed
    to by this pointer.
*/
class SaveObject: public PrintObject
{
public:
  SaveObject() { objectSaved = true; }  ///< Constructor.
  virtual ~SaveObject() {}  ///< Destructor.

  /** Has the object been saved?
   */
  virtual bool saved() const { return(objectSaved); }

  /** Clear the error status.

   */
  void clearError() { errStat = ""; }
  /** Get the object's error status.

      \return A description of the error.  If
      there is no error, the description is empty
  */
  const string getError() const { return(errStat); }

protected:

  /// Write to an XML element.
  virtual GdomeElement *writeXML(GdomeDocument *) = 0;

  /// Read from an XML element.
  virtual bool readXML(GdomeElement *) = 0;
  string getTextFromElement(GdomeElement *, gchar *)
    throw(FileException);
  void setTextInElement(GdomeDocument *, GdomeElement *, gchar *, gchar *)
    throw(FileException);
  string getAttribute(GdomeElement *, gchar *)
    throw(FileException);
  void setAttribute(GdomeElement *, gchar *, gchar *)
    throw(FileException);

  /// Specify that some parts of the object have been saved.
  virtual void setSaved() { objectSaved = true; }

  /// Specify that the object has been changed.
  void setChanged() { objectSaved = false; }

private:

  /** True if some parts of the object have been saved.

      A newly-created object has been saved, because it has no
      information content.  An object just read from disk is also
      saved.

      If an object is changed, then it has not been saved.

      Example: Changing the object description would make saved false.
  */
  bool objectSaved;
  /** Description of error.

      If a method encounters an error, it loads errStat with text describing
      the error.  If there is no error, errStat is loaded with "".
  */
  string errStat; 
};

#endif
