/*
 * $Id: support.c,v 1.2 2003/01/25 13:59:12 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include "support.h"

/* The original version of this function 
 * is taken from gtk+/tests/testtext.c 
 */
gint
text_view_line_numbers_expose_cb(GtkWidget      *widget,
				 GdkEventExpose *event,
				 GtkTextBuffer  *buf)
{
	gint count;
	GArray *numbers;
	GArray *pixels;
	gint first_y;
	gint last_y;
	gint i;
	GdkWindow *left_win;
	GdkWindow *right_win;
	PangoLayout *layout;
	GtkTextView *text_view;
	GtkTextWindowType type;
	GdkDrawable *target;
  	gint layout_width;
	gchar *str;
	static int num = 0;

	text_view = GTK_TEXT_VIEW (widget);
  
	/* See if this expose is on the line numbers window */
	left_win = gtk_text_view_get_window (text_view,
        				     GTK_TEXT_WINDOW_LEFT);
	right_win = gtk_text_view_get_window (text_view,
					      GTK_TEXT_WINDOW_RIGHT);

	if (event->window == left_win)
	{
		type = GTK_TEXT_WINDOW_LEFT;
		target = left_win;
  	}
  	else
		return FALSE;
  
	first_y = event->area.y;
	last_y = first_y + event->area.height;

	gtk_text_view_window_to_buffer_coords (text_view,
					       type,
					       0,
					       first_y,
					       NULL,
					       &first_y);

	gtk_text_view_window_to_buffer_coords (text_view,
					       type,
					       0,
					       last_y,
					       NULL,
					       &last_y);

	numbers = g_array_new (FALSE, FALSE, sizeof (gint));
	pixels = g_array_new (FALSE, FALSE, sizeof (gint));
  
	text_view_get_lines (text_view,
			     first_y,
			     last_y,
			     pixels,
			     numbers,
			     &count);
  
	layout = gtk_widget_create_pango_layout (widget, "");
  
	/* Set size.  get_line_count:  use gtk_text_buffer_get_line_count. */
	str = g_strdup_printf ("%d", MAX(99, gtk_text_buffer_get_line_count(GTK_TEXT_BUFFER (buf))));
	pango_layout_set_text (layout, str, -1);
	g_free (str);

	pango_layout_get_pixel_size (layout, &layout_width, NULL);

	gtk_text_view_set_border_window_size (GTK_TEXT_VIEW (text_view),
					      GTK_TEXT_WINDOW_LEFT,
					      layout_width + 4);
	
	/* Draw fully internationalized numbers! */	

	i = 0;
	while (i < count)
	{
		gint pos;
      
		gtk_text_view_buffer_to_window_coords (text_view,
						       type,
						       0,
						       g_array_index (pixels, gint, i),
						       NULL,
						       &pos);


		str = g_strdup_printf ("%d", g_array_index (numbers, gint, i) + 1);

		pango_layout_set_text (layout, str, -1);
		gtk_paint_layout (widget->style,
				  target,
				  GTK_WIDGET_STATE(widget),
				  FALSE,
				  NULL,
				  widget,
				  NULL,
				  2, pos,
				  layout);

		g_free (str);
      
		++i;
	}

	g_array_free (pixels, TRUE);
	g_array_free (numbers, TRUE);
  
	g_object_unref (G_OBJECT (layout));

	return TRUE;
}

/* This function is taken from gtk+/tests/testtext.c */
void
text_view_get_lines (GtkTextView  *text_view,
		     gint          first_y,
		     gint          last_y,
		     GArray       *buffer_coords,
		     GArray       *numbers,
		     gint         *countp)
{
	GtkTextIter iter;
	gint count;
	gint size;
      	gint last_line_num;	

	g_array_set_size (buffer_coords, 0);
	g_array_set_size (numbers, 0);
  
	/* Get iter at first y */
	gtk_text_view_get_line_at_y (text_view, &iter, first_y, NULL);

	/* For each iter, get its location and add it to the arrays.
	 * Stop when we pass last_y
	*/
	count = 0;
  	size = 0;

  	while (!gtk_text_iter_is_end (&iter))
    	{
		gint y, height;
      
		gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);

		g_array_append_val (buffer_coords, y);
		last_line_num = gtk_text_iter_get_line (&iter);
		g_array_append_val (numbers, last_line_num);
      	
		++count;

		if ((y + height) > last_y)
			break;
      
		gtk_text_iter_forward_line (&iter);
	}

	if (gtk_text_iter_is_end (&iter))
    	{
		gint y, height;
		gint line_num;
      
		gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);

		line_num = gtk_text_iter_get_line (&iter);

		if (line_num != last_line_num)
		{
			g_array_append_val (buffer_coords, y);
			g_array_append_val (numbers, line_num);
			++count;
		}
	}

	*countp = count;
}
